/* $Header: /plx/source/Xclients/generic/RCS/error.c,v 1.3 1996/01/30 20:49:01 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: error.c,v $
 * Revision 1.3  1996/01/30  20:49:01  lovvik
 *   Added some error code for the image library.
 *
 * Revision 1.2  1995/11/15  02:54:09  lovvik
 * Added error codes.  This allows the application to exit on fatal
 * errors.
 *
 * Revision 1.1  1995/10/02  23:41:43  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/error.c,v 1.3 1996/01/30 20:49:01 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/TextF.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>

#include "interface.h"
#include "globals.h"
#include "structs.h"
#include "prototypes.h"

static int errorCode = NO_ERROR;
extern char* client_name;

void PrintError(char *errorMessage, int error)
{
  XmString errorString;

  /*
   * Record all errors in the diagnostics log.
   */
  PrintDiagnostics(errorMessage);
  errorString = XmStringCreateSimple(errorMessage);
  XtVaSetValues(errorMessageLabel,
		XmNlabelString, errorString,
		NULL);
  XmStringFree(errorString);
  XtManageChild(errorDialog);
  XtPopup(errorShell, XtGrabExclusive);
  if(errorCode != ERROR_FATAL)
  {
    errorCode = error;
  }
  if(error == ERROR_FATAL)
  {
    /*
     * If this was a fatal error, show the help dialog box.
     */
    XtManageChild(helpDialog);
  }
}

int GetErrorCode(void)
{
  return(errorCode);
}


void ClearErrorCode(void)
{
  errorCode = NO_ERROR;
}


/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   E r r o r                                                                 %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function Error displays an error message and then terminates the program.
%
%  The format of the Error routine is:
%
%      Error(message,qualifier)
%
%  A description of each parameter follows:
%
%    o message: Specifies the message to display before terminating the
%      program.
%
%    o qualifier: Specifies any qualifier to the message.
%
%
*/
void Error(message,qualifier)
char
  *message,
  *qualifier;
{
  (void) fprintf(stderr,"%s: %s",client_name,message);
  if (qualifier != (char *) NULL)
    (void) fprintf(stderr," (%s)",qualifier);
  (void) fprintf(stderr,".\n");
  exit(1);
}
