/* $Header: /plx/source/Xclients/generic/RCS/help.c,v 1.4 1996/02/12 19:40:13 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: help.c,v $
 * Revision 1.4  1996/02/12  19:40:13  lovvik
 *   Modified InitializeHelp() to generate an error message if the
 *   PARALLAX_HOME environment variable is not set.  Also, added code
 *   to disable the SaveStill help button on an initialize error.
 *
 * Revision 1.3  1995/11/15  02:52:17  lovvik
 * Added some buttons to the DisableHelp function.
 *
 * Revision 1.2  1995/11/01  03:10:24  lovvik
 * Fixed compiler warnings.
 *
 * Revision 1.1  1995/10/02  23:43:27  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/help.c,v 1.4 1996/02/12 19:40:13 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/TextF.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>
#include "interface.h"
#include "structs.h"
#include "prototypes.h"

#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

static int helpPages = 0;
static int helpIndex[100];
static char *helpFilePath = NULL;
static FILE *helpFile;

void InitializeHelp(void)
{
  /*
   * Generate a help index by looking for C-l characters.
   */

  char *parallaxHomePath;
  
  parallaxHomePath = getenv("PARALLAX_HOME");
  if(!parallaxHomePath)
  {
    PrintError("Your PARALLAX_HOME environment variable is not set.  No help available.",
	       ERROR_NONFATAL);
    PrintDiagnostics("    To fix this, quit this application, set your PARALLAX_HOME");
    PrintDiagnostics("    environment variable, and restart this application.");
    DisableHelp();
    return;
  }

  helpFilePath = (char*)malloc(sizeof(char) * (strlen(parallaxHomePath) + 1 + 30));
  if(!helpFilePath)
  {
    PrintDiagnostics("Error allocating memory.");
    DisableHelp();
    return;
  }
  sprintf(helpFilePath, "%s/lib/Help/RTMovieTool.hlp", parallaxHomePath);
  if(OpenHelpFile())
  {
    char helpCharacter;
    
    /*
     * The first offset for help is at the beginning of the file.
     */
    helpPages = 0;
    helpIndex[helpPages] = 0;
      
    while(!feof(helpFile))
    {
      helpCharacter = fgetc(helpFile);
      
      /*
       * If the character we read was a new page (C-l),
       * then this is the end of the current help page.
       */
      if(helpCharacter == 12)
      {
	helpPages++;
	helpIndex[helpPages] = ftell(helpFile);
      }
    }

    fclose(helpFile);
  }
}


void DisableHelp(void)
{
  XtSetSensitive(helpOverviewButton, False);
  XtSetSensitive(helpRecordingButton, False);
  XtSetSensitive(helpPlayingButton, False);
  XtSetSensitive(helpSaveStillButton, False);
  XtSetSensitive(helpRecordQualityButton, False);
  XtSetSensitive(helpAudioButton, False);
  XtSetSensitive(helpRequirementsButton, False);
  XtSetSensitive(helpSettingsButton, False);
  XtSetSensitive(helpErrorsButton, False);
}


int OpenHelpFile(void)
{
  /*
   * If the file opens correctly, this function returns
   * True.  If any error occurs, this function will
   * return False.
   */
  if(!helpFilePath)
  {
    DisableHelp();
    return(False);
  }
  
  helpFile = fopen(helpFilePath, "r");
  if(!helpFile)
  {
    PrintError("Error opening help file.", ERROR_NONFATAL);
    DisableHelp();
    return(False);
  }
  return(True);
}


void DisplayHelp(int pageNumber)
{
  /*
   * Verify that the help page requested is a valid
   * page.
   */
  if(pageNumber < 0 || pageNumber >= helpPages)
  {
    return;
  }
  
  /*
   * Display the help page.  This is done by deleting all
   * current entries in the help list widget, then reading
   * the corresponding lines from the .hlp file into the
   * list widget.
   */
  XmListDeleteAllItems(helpList);
  if(OpenHelpFile())
  {
    char helpLine[100];
    int done = False;
    
    /*
     * Jump to the beginning of the help page in the help file.
     */
    fseek(helpFile, helpIndex[pageNumber], SEEK_SET);
    
    /*
     * Now, we are ready to display the text.  Get the first
     * line and see if it is '\n'.  If it is, then discard
     * it.
     */
    fgets(helpLine, 100, helpFile);
    if(helpLine[0] == '\n')
    {
      fgets(helpLine, 100, helpFile);
    }
    
    while(!feof(helpFile) && !done)
    {
      XmString helpLineString;
      
      if(ftell(helpFile) < helpIndex[pageNumber+1])
      {
	CleanHelpLine(helpLine);
	helpLineString = XmStringCreateSimple(helpLine);
	XmListAddItemUnselected(helpList, helpLineString, 0);
	XmStringFree(helpLineString);

	fgets(helpLine, 100, helpFile);
      }
      else
      {
	done = True;
      }
    }
    fclose(helpFile);
  }
}


void CleanHelpLine(char *helpLine)
{
  int position;
  
  for(position = 0; position < strlen(helpLine); position++)
  {
    if(helpLine[position] == '\n')
    {
      helpLine[position] = '\0';
    }
    if(helpLine[position] == '\t')
    {
      helpLine[position] = ' ';
    }
  }
}


