/* $Header: /plx/source/Xclients/generic/RCS/mainDialog.c,v 1.10 1996/01/30 20:43:37 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: mainDialog.c,v $
 * Revision 1.10  1996/01/30  20:43:37  lovvik
 *   Added SetImageFormat() and GetImageFormat() for
 *   the still image save feature.
 *
 * Revision 1.9  1996/01/11  18:16:10  lovvik
 *   Added GetOutputParameters() and SetOutputParameters().
 *
 * Revision 1.8  1996/01/04  18:33:45  lovvik
 * Made the input cascade insensitive while a movie is loaded.
 * If a movie is loaded, Rtv requires that the input be set to
 * the same standard (NTSC or PAL) as the movie.  If the standard
 * is different than the movie, the movie will not play correctly.
 *
 * Revision 1.7  1995/12/14  17:25:46  lovvik
 * Changed PostInitializeWorkProc().  This function required that
 * the recordWidget and playbackWidget have windows when it was
 * called.  An XDesigner upgrade broke this code.  For some reason,
 * the new code did not realize all of the top-level shells.  I
 * simply put the XtRealizeWidget() calls in this function to
 * guarantee that the widgets have valid windows before the rest
 * of the function is executed.
 *
 * Revision 1.6  1995/11/20  20:37:07  lovvik
 * Took out the call to SetFramesPerSecond() in the InitializeMovieTool()
 * function.  This call is made in InitializeVideo() (in video.c).
 *
 * Revision 1.5  1995/11/15  02:50:35  lovvik
 * Added code to catch signals.  This allows the application to quit
 * gracefully in the event of a kill or Ctrl-C.
 *
 * Revision 1.4  1995/11/01  02:46:17  lovvik
 * Added RefreshCurrentFrame().  This function allows the playback window
 * to be refreshed after an expose event.  (Called from window.c).
 *
 * Revision 1.3  1995/10/12  20:07:49  lovvik
 * Added GetMode() and SetMode() for frame and field support.
 *
 * Revision 1.2  1995/10/11  17:07:36  lovvik
 * Added a call to SetRTVfps() in the SetFramesPerSecond()
 * function.  Also added a call to SetRTVsize() in the
 * SetCurrentMovieSize() function.
 *
 * Revision 1.1  1995/10/02  23:45:25  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/mainDialog.c,v 1.10 1996/01/30 20:43:37 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>

#include <XPlxExt.h>

#include "interface.h"
#include "structs.h"
#include "globals.h"
#include "prototypes.h"

static MovieToolStruct movieTool;

void InitializeMovieTool(void)
{
  SetMovieToolStatus(NO_MOVIE);
  movieTool.movieLoaded = False;
  movieTool.movieFileName = NULL;
  movieTool.movieLengthInFrames = 0;

  movieTool.currentFrame = 0;
  movieTool.outputStatus = False;
  movieTool.inputStatus = False;
  movieTool.inputParameters.channelToUse = PLX_INPUT_0;
  movieTool.inputParameters.inputSignalStandard = PLX_NTSC;
  movieTool.inputParameters.inputSignalFormat = PLX_COMP;
  movieTool.inputParameters.inputInstance = 0;
  movieTool.inputParameters.inputWidth = 640;
  movieTool.inputParameters.inputHeight = 480;
  movieTool.outputParameters.outputSignalFormat = PLX_IO_OFF;
  movieTool.liveVideoSize.width = 0;
  movieTool.liveVideoSize.height = 0;
  movieTool.currentMovie.width = 0;
  movieTool.currentMovie.height = 0;
  movieTool.mode = FRAME_MODE;
  strcpy(movieTool.imageFormat, "SUN");

  InitializeJPEGCompressionDialog();

  InitializeRTV();
  ConnectRTV();

  InitializeVideoWidget();
  BuildInputMenu();
  InitializeColorSettingsDialog();
  InitializeAboutBox();
  InitializeHelp();
  InitializeDiagnostics();

  SetMovieToolTitle();

  InitializeAudioDialog();
  InitializeWindowModule();
  DisableNonimplementedFeatures();
  SetMovieToolStatus(NO_MOVIE);
  XtAppAddWorkProc(app_context,
		   PostInitializeWorkProc,
		   (XtPointer)NULL);
  CatchSignals();
}


Boolean PostInitializeWorkProc(XtPointer unusedClientData)
{
  XtRealizeWidget(input1Shell);
  XtRealizeWidget(movieShell);
  rtvGC = XCreateGC(XtDisplay(recordWidget), XtWindow(recordWidget),
		    0, 0);
#ifndef DEBUG
/*
 *  Tell X-Server to connect to RTV server via protocol request.
 *  Connections are made for record and playback windows.
 */
  XPlxSetRtvWindow(XtDisplay(recordWidget),
		   XtWindow(recordWidget),
		   rtvGC);
  XPlxSetRtvWindow(XtDisplay(playbackWidget),
		   XtWindow(playbackWidget),
		   rtvGC);
#endif

  PositionWindows();
  XtPopup(input1Shell, XtGrabNone);  
  return(True);
}



int GetMovieToolStatus(void)
{
  return(movieTool.status);
}


void SetMovieToolStatus(int status)
{
  movieTool.status = status;

  /* Disable/enable components here */
  switch(movieTool.status)
  {
    case NO_MOVIE:
    {
      XtSetSensitive(fileMenuCascade, True);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      XtSetSensitive(saveStillMenuButton, False);
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(editMenuCascade, False);

      XtSetSensitive(inputMenuCascade, True);

      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, True);
      
      if(GetOutputAvailable() == True)
      {
	XtSetSensitive(outputCascadeButton, False);
      }

      XtSetSensitive(helpMenuCascade, True);

      XtSetSensitive(hourPlusButton, False);
      XtSetSensitive(hourMinusButton, False);
      XtSetSensitive(minutePlusButton, False);
      XtSetSensitive(minuteMinusButton, False);
      XtSetSensitive(secondPlusButton, False);
      XtSetSensitive(secondMinusButton, False);
      XtSetSensitive(framePlusButton, False);
      XtSetSensitive(frameMinusButton, False);
      
      XtSetSensitive(searchSlider, False);
      XtSetSensitive(recordButton, False);
      XtSetSensitive(pauseButton, False);
      XtSetSensitive(playButton, False);
      XtSetSensitive(stopButton, False);
      XtSetSensitive(startOfMovieButton, False);
      XtSetSensitive(frameReverseButton, False);
      XtSetSensitive(frameAdvanceButton, False);
      XtSetSensitive(endOfMovieButton, False);

      XtSetSensitive(fullSizeButton, True);
      XtSetSensitive(quarterSizeButton, True);
      XtSetSensitive(imageQualitySlider, True);
      XtSetSensitive(qfactorSlider, True);

      XtSetSensitive(audioOnToggle, True);
      XtSetSensitive(audioOffToggle, True);

      break;
    }
    
    case MT_IDLE:
    {
      XtSetSensitive(fileMenuCascade, True);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      if(GetMovieLoaded())
      {
	XtSetSensitive(saveStillMenuButton, True);
	XtSetSensitive(editMenuCascade, True);
      }
      else
      {
	XtSetSensitive(saveStillMenuButton, False);
	XtSetSensitive(editMenuCascade, False);
      }	
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, True);
      XtSetSensitive(helpMenuCascade, True);

      XtSetSensitive(pauseButton, False);
      if(GetMovieLoaded())
      {
	XtSetSensitive(inputMenuCascade, False);

	if(GetOutputAvailable() == True)
	{
	  XtSetSensitive(outputCascadeButton, True);
	}

	XtSetSensitive(playButton, True);
	XtSetSensitive(hourPlusButton, True);
	XtSetSensitive(hourMinusButton, True);
	XtSetSensitive(minutePlusButton, True);
	XtSetSensitive(minuteMinusButton, True);
	XtSetSensitive(secondPlusButton, True);
	XtSetSensitive(secondMinusButton, True);
	XtSetSensitive(framePlusButton, True);
	XtSetSensitive(frameMinusButton, True);
      
	XtSetSensitive(searchSlider, True);

	XtSetSensitive(startOfMovieButton, True);
	XtSetSensitive(frameReverseButton, True);
	XtSetSensitive(frameAdvanceButton, True);
	XtSetSensitive(endOfMovieButton, True);

	XtSetSensitive(fullSizeButton, False);
	XtSetSensitive(quarterSizeButton, False);
	XtSetSensitive(imageQualitySlider, False);
	XtSetSensitive(qfactorSlider, False);
      }
      else
      {
	XtSetSensitive(inputMenuCascade, True);

	if(GetOutputAvailable() == True)
	{
	  XtSetSensitive(outputCascadeButton, False);
	}
	XtSetSensitive(recordButton, True);
	XtSetSensitive(hourPlusButton, False);
	XtSetSensitive(hourMinusButton, False);
	XtSetSensitive(minutePlusButton, False);
	XtSetSensitive(minuteMinusButton, False);
	XtSetSensitive(secondPlusButton, False);
	XtSetSensitive(secondMinusButton, False);
	XtSetSensitive(framePlusButton, False);
	XtSetSensitive(frameMinusButton, False);
      
	XtSetSensitive(searchSlider, False);

	XtSetSensitive(startOfMovieButton, False);
	XtSetSensitive(frameReverseButton, False);
	XtSetSensitive(frameAdvanceButton, False);
	XtSetSensitive(endOfMovieButton, False);

	XtSetSensitive(fullSizeButton, True);
	XtSetSensitive(quarterSizeButton, True);
	XtSetSensitive(imageQualitySlider, True);
	XtSetSensitive(qfactorSlider, True);
      }

      XtSetSensitive(stopButton, False);

      XtSetSensitive(audioOnToggle, True);
      XtSetSensitive(audioOffToggle, True);

      break;
    }

    case MT_PLAY_PAUSE:
    {
      XtSetSensitive(fileMenuCascade, False);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      XtSetSensitive(saveStillMenuButton, True);
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(inputMenuCascade, False);

      XtSetSensitive(editMenuCascade, False);

      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, False);
      if(GetOutputAvailable() == True)
      {
	XtSetSensitive(outputCascadeButton, True);
      }

      XtSetSensitive(helpMenuCascade, True);

      XtSetSensitive(hourPlusButton, False);
      XtSetSensitive(hourMinusButton, False);
      XtSetSensitive(minutePlusButton, False);
      XtSetSensitive(minuteMinusButton, False);
      XtSetSensitive(secondPlusButton, False);
      XtSetSensitive(secondMinusButton, False);
      XtSetSensitive(framePlusButton, False);
      XtSetSensitive(frameMinusButton, False);
      
      XtSetSensitive(searchSlider, False);
      XtSetSensitive(recordButton, False);
      XtSetSensitive(pauseButton, True);
      XtSetSensitive(playButton, True);
      XtSetSensitive(stopButton, True);
      XtSetSensitive(startOfMovieButton, False);
      XtSetSensitive(frameReverseButton, False);
      XtSetSensitive(frameAdvanceButton, False);
      XtSetSensitive(endOfMovieButton, False);

      XtSetSensitive(fullSizeButton, False);
      XtSetSensitive(quarterSizeButton, False);
      XtSetSensitive(imageQualitySlider, False);
      XtSetSensitive(qfactorSlider, False);

      XtSetSensitive(audioOnToggle, False);
      XtSetSensitive(audioOffToggle, False);

      break;
    }

    case MT_RECORD_PAUSE:
    {
      XtSetSensitive(fileMenuCascade, False);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      XtSetSensitive(saveStillMenuButton, False);
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(editMenuCascade, False);

      XtSetSensitive(inputMenuCascade, False);

      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, False);
      if(GetOutputAvailable() == True)
      {
	XtSetSensitive(outputCascadeButton, False);
      }

      XtSetSensitive(helpMenuCascade, True);

      XtSetSensitive(hourPlusButton, False);
      XtSetSensitive(hourMinusButton, False);
      XtSetSensitive(minutePlusButton, False);
      XtSetSensitive(minuteMinusButton, False);
      XtSetSensitive(secondPlusButton, False);
      XtSetSensitive(secondMinusButton, False);
      XtSetSensitive(framePlusButton, False);
      XtSetSensitive(frameMinusButton, False);
      
      XtSetSensitive(searchSlider, False);
      XtSetSensitive(recordButton, True);
      XtSetSensitive(pauseButton, True);
      XtSetSensitive(playButton, False);
      XtSetSensitive(stopButton, True);
      XtSetSensitive(startOfMovieButton, False);
      XtSetSensitive(frameReverseButton, False);
      XtSetSensitive(frameAdvanceButton, False);
      XtSetSensitive(endOfMovieButton, False);

      XtSetSensitive(fullSizeButton, False);
      XtSetSensitive(quarterSizeButton, False);
      XtSetSensitive(imageQualitySlider, False);
      XtSetSensitive(qfactorSlider, False);

      XtSetSensitive(audioOnToggle, False);
      XtSetSensitive(audioOffToggle, False);

      break;
    }
      
    case MT_PLAY:
    {
      XtSetSensitive(fileMenuCascade, False);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      XtSetSensitive(saveStillMenuButton, False);
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(editMenuCascade, False);

      XtSetSensitive(inputMenuCascade, False);

      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, False);
      if(GetOutputAvailable() == True)
      {
	XtSetSensitive(outputCascadeButton, True);
      }

      XtSetSensitive(helpMenuCascade, False);

      XtSetSensitive(hourPlusButton, False);
      XtSetSensitive(hourMinusButton, False);
      XtSetSensitive(minutePlusButton, False);
      XtSetSensitive(minuteMinusButton, False);
      XtSetSensitive(secondPlusButton, False);
      XtSetSensitive(secondMinusButton, False);
      XtSetSensitive(framePlusButton, False);
      XtSetSensitive(frameMinusButton, False);

      XtSetSensitive(searchSlider, True);
      XtSetSensitive(recordButton, False);
      XtSetSensitive(pauseButton, True);
      XtSetSensitive(playButton, False);
      XtSetSensitive(stopButton, True);
      XtSetSensitive(startOfMovieButton, False);
      XtSetSensitive(frameReverseButton, False);
      XtSetSensitive(frameAdvanceButton, False);
      XtSetSensitive(endOfMovieButton, False);

      XtSetSensitive(fullSizeButton, False);
      XtSetSensitive(quarterSizeButton, False);
      XtSetSensitive(imageQualitySlider, False);
      XtSetSensitive(qfactorSlider, False);

      XtSetSensitive(audioOnToggle, False);
      XtSetSensitive(audioOffToggle, False);

      break;
    }   

    case MT_RECORD:
    {
      XtSetSensitive(fileMenuCascade, False);
      XtSetSensitive(openMenuButton, True);
      XtSetSensitive(saveAsMenuButton, True);
      XtSetSensitive(saveStillMenuButton, False);
      XtSetSensitive(quitMenuButton, True);
      
      XtSetSensitive(editMenuCascade, False);

      XtSetSensitive(inputMenuCascade, False);

      XtSetSensitive(optionsMenuCascade, True);
      XtSetSensitive(smartMoviesMenuButton, True);
      XtSetSensitive(audioMenuButton, True);
      XtSetSensitive(modeMenuCascade, False);
      if(GetOutputAvailable() == True)
      {
	XtSetSensitive(outputCascadeButton, False);
      }

      XtSetSensitive(helpMenuCascade, False);

      XtSetSensitive(hourPlusButton, False);
      XtSetSensitive(hourMinusButton, False);
      XtSetSensitive(minutePlusButton, False);
      XtSetSensitive(minuteMinusButton, False);
      XtSetSensitive(secondPlusButton, False);
      XtSetSensitive(secondMinusButton, False);
      XtSetSensitive(framePlusButton, False);
      XtSetSensitive(frameMinusButton, False);

      XtSetSensitive(searchSlider, False);
      XtSetSensitive(recordButton, False);
      XtSetSensitive(pauseButton, True);
      XtSetSensitive(playButton, False);
      XtSetSensitive(stopButton, True);
      XtSetSensitive(startOfMovieButton, False);
      XtSetSensitive(frameReverseButton, False);
      XtSetSensitive(frameAdvanceButton, False);
      XtSetSensitive(endOfMovieButton, False);
      
      XtSetSensitive(fullSizeButton, False);
      XtSetSensitive(quarterSizeButton, False);
      XtSetSensitive(imageQualitySlider, False);
      XtSetSensitive(qfactorSlider, False);

      XtSetSensitive(audioOnToggle, False);
      XtSetSensitive(audioOffToggle, False);

      break;
    }

    default:
    {
      PrintDiagnostics("Warning:  SetMovieToolStatus called with bad parameter.");
    }      
  }
}


int GetLiveVideoStatus(void)
{
  return(movieTool.liveVideo);
}


void SetLiveVideoStatus(int status)
{
  movieTool.liveVideo = status;
}

int GetMovieLoaded(void)
{
  return(movieTool.movieLoaded);
}


void SetMovieLoaded(int loaded)
{
  if(movieTool.movieLoaded != loaded)
  {
    movieTool.movieLoaded = loaded;
    if(movieTool.movieLoaded == True)
    {
      SetMovieToolStatus(MT_IDLE);
    }
    else
    {
      SetMovieToolStatus(NO_MOVIE);
    }
  }
}


char *GetMovieFileName(void)
{
  return(movieTool.movieFileName);
}


void SetMovieFileName(char *fileName)
{
  if(fileName != NULL)
  {
    if(movieTool.movieFileName != NULL)
    {
      free(movieTool.movieFileName);
    }
    
    movieTool.movieFileName = (char*)malloc(sizeof(char) * (strlen(fileName) + 1));
    if(movieTool.movieFileName != NULL)
    {
      strcpy(movieTool.movieFileName, fileName);
    }
    /*
     * Since we are assigning a new filename, we should invalidate the
     * current frame.  This way, we can be sure that SetCurrentFrame()
     * will display the new frame rather than just set up the user interface.
     */
    movieTool.currentFrame = -1;
  }  
  else
  {
    if(movieTool.movieFileName != NULL)
    {
      free(movieTool.movieFileName);
      movieTool.movieFileName = NULL;
    }
  }
  SetMovieToolTitle();
}


int GetMovieLengthInFrames(void)
{
  return(movieTool.movieLengthInFrames);
}


void SetMovieLengthInFrames(int length)
{
  char endTime[13];
  char cHours[3];
  char cMinutes[3];
  char cSeconds[3];
  char cFrames[3];
  int hours;
  int minutes;
  int seconds;
  int frames;
  XmString endTimeXmString;
  
  if(movieTool.movieLengthInFrames != length)
  {
    movieTool.movieLengthInFrames = length;
    XmScaleSetValue(searchSlider, 0);
    hours = FrameHours(length, GetFramesPerSecond());
    if(hours < 10)
    {
      sprintf(cHours, "0%d", hours);
    }
    else
    {
      sprintf(cHours, "%d", hours);
    }
    
    minutes = FrameMinutes(length, GetFramesPerSecond());
    if(minutes < 10)
    {
      sprintf(cMinutes, "0%d", minutes);
    }
    else
    {
      sprintf(cMinutes, "%d", minutes);
    }
    
    seconds = FrameSeconds(length, GetFramesPerSecond());
    if(seconds < 10)
    {
      sprintf(cSeconds, "0%d", seconds);
    }
    else
    {
      sprintf(cSeconds, "%d", seconds);
    }
    
    frames = FrameFrames(length, GetFramesPerSecond());
    if(frames < 10)
    {  
      sprintf(cFrames, "0%d", frames);
    }
    else
    {
      sprintf(cFrames, "%d", frames);
    }
    
    sprintf(endTime, "%s:%s:%s:%s", cHours, cMinutes, cSeconds, cFrames);
    endTimeXmString = XmStringCreate(endTime, XmFONTLIST_DEFAULT_TAG);

    XtVaSetValues(endTimeLabel, 
		  XmNlabelString, endTimeXmString,
		  NULL);
    XmStringFree(endTimeXmString);
    if(movieTool.currentFrame != -1)
    {
      SetTimerCurrentFrame(movieTool.currentFrame, movieTool.framesPerSecond);
    }
    else
    {
      SetTimerCurrentFrame(0, movieTool.framesPerSecond);
    }
  }
}


int GetFramesPerSecond(void)
{
  return(movieTool.framesPerSecond);
}


void SetFramesPerSecond(int fps)
{
  static int initialized = False;
  
  char framesPerSecond[3];
  
  if(movieTool.framesPerSecond != fps || initialized == False)
  {
    movieTool.framesPerSecond = fps;

    if(fps < 10)
    {
      sprintf(framesPerSecond, "0%d", fps);
    }
    else
    {
      sprintf(framesPerSecond, "%d", fps);
    }
    SetRTVfps(fps);
    XmTextSetString(framesPerSecondText, framesPerSecond);
    initialized = True;
  }
}


int GetCurrentFrame(void)
{
  return(movieTool.currentFrame);
}


void SetCurrentFrame(int currentFrame)
{
  int mode;
  
  /*
   * If there is no change in the frame, do
   * not process this request.
   */
  if(currentFrame == movieTool.currentFrame)
  {
    return;
  }
  
  if(currentFrame <= movieTool.movieLengthInFrames)
  {

    SetTimerCurrentFrame(currentFrame, movieTool.framesPerSecond);

    if(movieTool.movieLengthInFrames != 0)
    {
      XmScaleSetValue(searchSlider, (int)(((float)currentFrame /
					   (float)movieTool.movieLengthInFrames) *
					  100));
    }
    else
    {
      XmScaleSetValue(searchSlider, 0);
    }
    mode = GetMode();
    if(mode == FIELD_MODE)
    {
      RTVDisplayOneFrame(currentFrame, movieTool.currentFrame, True);
    }
    else
    {
      RTVDisplayOneFrame(currentFrame, movieTool.currentFrame, False);
    }
    movieTool.currentFrame = currentFrame;
  }
}


void SetCurrentFrameNoDisplay(int currentFrame)
{
  if(currentFrame <= movieTool.movieLengthInFrames)
  {
    movieTool.currentFrame = currentFrame;
    SetTimerCurrentFrame(movieTool.currentFrame, movieTool.framesPerSecond);

    if(movieTool.movieLengthInFrames != 0)
    {
      XmScaleSetValue(searchSlider, (int)(((float)movieTool.currentFrame /
					   (float)movieTool.movieLengthInFrames) *
					  100));
    }
    else
    {
      XmScaleSetValue(searchSlider, 0);
    }
  }
}


void SynchronizeFrames(void)
{
  int currentFrame;
  int mode;
  
  if(!RTVGetCurrentFrame(&currentFrame))
  {
    PrintDiagnostics("Could not synchronize with the rtv server.");
    return;
  }
  
  mode = GetMode();
  if(mode == FIELD_MODE)
  {
    currentFrame *= 2;
  }
  
  if(currentFrame != GetCurrentFrame())
  {
    if(currentFrame <= movieTool.movieLengthInFrames)
    {
      movieTool.currentFrame = currentFrame;
      SetTimerCurrentFrame(movieTool.currentFrame, movieTool.framesPerSecond);

      if(movieTool.movieLengthInFrames != 0)
      {
	XmScaleSetValue(searchSlider, (int)(((float)movieTool.currentFrame /
					     (float)movieTool.movieLengthInFrames) *
					    100));
      }
      else
      {
	XmScaleSetValue(searchSlider, 0);
      }
    }
  }
}


void SetCurrentFrameWithSlider(int currentFrame)
{
  /*
   * Note:  Displaying a frame with the search slider is different in that
   * you could actually end up displaying more than 30 frames per second.
   * To make sure that the driver can handle the number of requests that
   * can possibly come out of a slider movement, another call was developed
   * in the RTV api.  Be sure to use this call instead, or the rtv driver
   * could crash.
   */
  if(currentFrame <= movieTool.movieLengthInFrames)
  {
    movieTool.currentFrame = currentFrame;
    SetTimerCurrentFrame(movieTool.currentFrame, movieTool.framesPerSecond);

    if(movieTool.movieLengthInFrames != 0)
    {
      XmScaleSetValue(searchSlider, (int)(((float)movieTool.currentFrame /
					   (float)movieTool.movieLengthInFrames) *
					  100));
    }
    else
    {
      XmScaleSetValue(searchSlider, 0);
    }
    if(movieTool.mode == FIELD_MODE)
    {
      RTVDisplayOneFrameWithSlider(movieTool.currentFrame, True);
    }
    else
    {
      RTVDisplayOneFrameWithSlider(movieTool.currentFrame, False);
    }
  }
}


int GetOutputStatus(void)
{
  return(movieTool.outputStatus);
}


void SetOutputStatus(int status)
{
  movieTool.outputStatus = status;
}


int GetInputStatus(void)
{
  return(movieTool.inputStatus);
}


void SetInputStatus(int status)
{
  movieTool.inputStatus = status;
}


int GetOutputAvailable(void)
{
  return(movieTool.outputAvailable);
}


void SetOutputAvailable(int status)
{
  movieTool.outputAvailable = status;
}


void GetInputParameters(InputParameters *inputParameters)
{
  inputParameters->channelToUse = movieTool.inputParameters.channelToUse;
  inputParameters->inputSignalStandard = movieTool.inputParameters.inputSignalStandard;
  inputParameters->inputSignalFormat = movieTool.inputParameters.inputSignalFormat;
  inputParameters->inputInstance = movieTool.inputParameters.inputInstance;
}


void SetInputParameters(InputParameters *inputParameters)
{
  movieTool.inputParameters.channelToUse = inputParameters->channelToUse;
  movieTool.inputParameters.inputSignalStandard = inputParameters->inputSignalStandard;
  movieTool.inputParameters.inputSignalFormat = inputParameters->inputSignalFormat;
  movieTool.inputParameters.inputInstance = inputParameters->inputInstance;
}


void GetOutputParameters(OutputParameters *outputParameters)
{
  outputParameters->outputSignalStandard = movieTool.outputParameters.outputSignalStandard;
  outputParameters->outputSignalFormat = movieTool.outputParameters.outputSignalFormat;
  outputParameters->outputSignalWidth = movieTool.outputParameters.outputSignalWidth;
  outputParameters->outputSignalHeight = movieTool.outputParameters.outputSignalHeight;
}

void SetOutputParameters(OutputParameters *outputParameters)
{
  movieTool.outputParameters.outputSignalStandard = outputParameters->outputSignalStandard;
  movieTool.outputParameters.outputSignalFormat = outputParameters->outputSignalFormat;
  movieTool.outputParameters.outputSignalWidth = outputParameters->outputSignalWidth;
  movieTool.outputParameters.outputSignalHeight = outputParameters->outputSignalHeight;  
}


int GetInputWidth(void)
{
  return(movieTool.inputParameters.inputWidth);
}


void SetInputWidth(int width)
{
  movieTool.inputParameters.inputWidth = width;
}


int GetInputHeight(void)
{
  return(movieTool.inputParameters.inputHeight);
}


void SetInputHeight(int height)
{
  movieTool.inputParameters.inputHeight = height;
}


void SetMovieToolTitle(void)
{
  char *title;
  char *applicationName;
  char *applicationStatus;
  char *movieShellTitle = NULL;

  applicationName = (char *)malloc(sizeof(char) *
				   (strlen("RTMovieTool: ") + 1));
  if(applicationName == NULL)
  {
    PrintDiagnostics("Error!  Could not allocate memory for the title!");
    return;
  }
  strcpy(applicationName, "RTMovieTool");

  switch(GetMovieToolStatus())
  {
    case MT_PLAY_PAUSE:
    case MT_RECORD_PAUSE:
    {
      applicationStatus = (char *)malloc(sizeof(char) *
					 (strlen("Paused") + 1));
      if(applicationStatus == NULL)
      {
	PrintDiagnostics("Error!  Could not allocate memory for the title!");
	free(applicationName);
	return;
      }
      strcpy(applicationStatus, "Paused");
      break;
    }

    case MT_PLAY:
    {
      applicationStatus = (char *)malloc(sizeof(char) *
					 (strlen("Playing") + 1));
      if(applicationStatus == NULL)
      {
	PrintDiagnostics("Error!  Could not allocate memory for the title!");
	free(applicationName);
	return;
      }
      strcpy(applicationStatus, "Playing");
      break;
    }

    case MT_RECORD:
    {
      applicationStatus = (char *)malloc(sizeof(char) *
					 (strlen("Recording") + 1));
      if(applicationStatus == NULL)
      {
	PrintDiagnostics("Error!  Could not allocate memory for the title!");
	free(applicationName);
	return;
      }
      strcpy(applicationStatus, "Recording");
      break;
    }

    default:
    {
      applicationStatus = (char *)malloc(sizeof(char) *
					 (strlen("Idle") + 1));
      if(applicationStatus == NULL)
      {
	PrintDiagnostics("Error!  Could not allocate memory for the title!");
	free(applicationName);
	return;
      }
      strcpy(applicationStatus, "Idle");
      break;
    }
  }

  if(GetMovieFileName() != NULL)
  {
    movieShellTitle = (char *)malloc(sizeof(char) * (strlen(GetMovieFileName()) + 1));
    if(!movieShellTitle)
    {
      PrintDiagnostics("Error!  Could not allocate memory for the title!");
      free(applicationName);
      free(applicationStatus);
      return;
    }
    strcpy(movieShellTitle, GetMovieFileName());    

  /*
   * Allocate memory for the new title.  We add 1 for the '\0' at the end of the
   * string.  5 is added to this length for formatting characters that we will
   * add to the title.
   */
    title = (char *)malloc(sizeof(char) *
			   (strlen(applicationName) +
			    strlen(applicationStatus) +
			    strlen(GetMovieFileName()) + 1 + 5));
  }

  else
  {
    title = (char *)malloc(sizeof(char) *
			   (strlen(applicationName) +
			    strlen(applicationStatus) + 1 + 5));
  }

  if(title == NULL)
  {
    PrintDiagnostics("Error!  Could not allocate memory for the title!");
    free(applicationName);
    free(applicationStatus);
    return;
  }

  if(GetMovieFileName() != NULL)
  {
    sprintf(title, "%s: %s - %s", 
	    applicationName, applicationStatus,
	    GetMovieFileName());
  }
  else
  {
    sprintf(title, "%s: %s -", 
	    applicationName, applicationStatus);
  }
  
  XtVaSetValues(mainWindow,
		XmNtitle, title,
		NULL);

  if(movieShellTitle != NULL)
  {
    XtVaSetValues(movieShell,
		  XmNtitle, movieShellTitle,
		  NULL);
    free(movieShellTitle);
  }
  
  free(title);
  free(applicationName);
  free(applicationStatus);
}


void GetLiveVideoSize(int *width, int *height)
{
  *width = movieTool.liveVideoSize.width;
  *height = movieTool.liveVideoSize.height;
}


void SetLiveVideoSize(int width, int height, int resize)
{
  char message[100];

  if(movieTool.liveVideoSize.width != width ||
     movieTool.liveVideoSize.height != height)
  {
    switch(GetMovieToolStatus())
    {
      case MT_PLAY:
      case MT_RECORD:
      case MT_PLAY_PAUSE:
      case MT_RECORD_PAUSE:
      {
	PrintDiagnostics("Window resized during bad state.");
	break;
      }

      case MT_IDLE:
      case NO_MOVIE:
      {  
	if(movieTool.liveVideoSize.width == 0 &&
	   movieTool.liveVideoSize.height == 0)
	{
	  /*
	   * Initialize the video width and height.
	   * (No need to print diagnostic message).
	   */
	  movieTool.liveVideoSize.width = width;
	  movieTool.liveVideoSize.height = height;

	  if(resize == True)
	  {
	    ResizeVideoWindow();
	  }
	}
	else
	{
	  movieTool.liveVideoSize.width = width;
	  movieTool.liveVideoSize.height = height;

	  if(resize == True)
	  {
	    ResizeVideoWindow();
	  }
	  sprintf(message, "Record window resized:  Width:%d, Height:%d", width, height);
	  PrintDiagnostics(message);
	}
      }
    }
  }
}

void GetCurrentMovieSize(int *width, int *height)
{
  *width = movieTool.currentMovie.width;
  *height = movieTool.currentMovie.height;
}


void SetCurrentMovieSize(int width, int height)
{
  char message[100];
  if(movieTool.currentMovie.width != width ||
     movieTool.currentMovie.height != height)
  {
    movieTool.currentMovie.width = width;
    movieTool.currentMovie.height = height;
    ResizePlaybackWindow();
    sprintf(message, "Playback window resized:  Width:%d, Height:%d", width, height);
    PrintDiagnostics(message);
  }
}


int GetMode(void)
{
  return(movieTool.mode);
}


void SetMode(int mode)
{
  movieTool.mode = mode;
}


void RefreshCurrentFrame(void)
{
  int currentFrame;
  int mode;
  
  currentFrame = movieTool.currentFrame;
  
  if(currentFrame <= movieTool.movieLengthInFrames)
  {
    mode = GetMode();
    if(mode == FIELD_MODE)
    {
      RTVDisplayOneFrame(currentFrame, movieTool.currentFrame, True);
    }
    else
    {
      RTVDisplayOneFrame(currentFrame, movieTool.currentFrame, False);
    }
  }
}


void SetImageFormat(char *format)
{
  strcpy(movieTool.imageFormat, format);
}

char* GetImageFormat(void)
{
  return(movieTool.imageFormat);
}
