/* $Header: /plx/source/Xclients/generic/RCS/rtvModule.c,v 1.10 1996/01/30 20:33:29 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: rtvModule.c,v $
 * Revision 1.10  1996/01/30  20:33:29  lovvik
 *   Fixed HandleRtvError() to reset the rtv
 *   status to IDLE on all errors.
 *
 *   Also, added three functions for movie
 *   editing:  RTVSetFirstValidFrame(),
 *   RTVSetLastValidFrame(), and
 *   RTVUseOriginalMovieLength().
 *
 * Revision 1.9  1996/01/04  18:03:26  lovvik
 * Changed the call to XPlxRtvPutCImage() to include
 * an argument for the signal standard.
 *
 * Revision 1.8  1995/11/27  22:29:36  lovvik
 * Removed the include file <rtv.h>.  This include file contains
 * Rtv server specific information, and should not be used in
 * client code.
 *
 * Revision 1.7  1995/11/20  20:42:33  lovvik
 * Removed a printf statement that was used for testing.
 *
 * Revision 1.6  1995/11/15  02:48:40  lovvik
 * Added error code.
 *
 * Revision 1.5  1995/11/01  02:38:41  lovvik
 * Added HandleRtvError().  Removed the bandwidth argument from
 * GetServerStatus().  Also, added RecordPause().
 *
 * Revision 1.4  1995/10/12  20:14:13  lovvik
 * Modified RTVDisplayOneFrame() to support field mode.  Also
 * added RTVWindowRecordPrepare() and RTVWindowPlayPrepare()
 * which tell the window server to paint tags for RTV.
 *
 * Revision 1.3  1995/10/11  17:06:22  lovvik
 * Added SetRTVsize(), SetRTVfps(), and filled in the
 * SetRTVPosition() function.
 *
 * Revision 1.2  1995/10/06  17:18:47  lovvik
 * Removed code that was necessary until server support
 * existed.
 *
 * Revision 1.1  1995/10/02  23:46:45  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/rtvModule.c,v 1.10 1996/01/30 20:33:29 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>

#include <XPlxExt.h>
#include <XVideoIn.h>

#include "globals.h"
#include "structs.h"
#include "prototypes.h"

#ifdef USE_RTV_CODE
#include <rtv_ops.h>
#else
typedef struct
{
  int audio_mode;
  int audio_status;
} RTV_OBJECT;
typedef struct
{
  int interlace;
  int win_x;
  int win_y;
  int win_w;
  int win_h;
  int fps;
  int qfactor;
  int rate;
} RTV_ATTRIBUTES;
#endif

#define RTV_MODULE_CONNECTED 1
#define RTV_MODULE_DISCONNECTED 2
#define RTV_MODULE_IDLE 3
#define RTV_MODULE_PLAY 4
#define RTV_MODULE_RECORD 5
#define RTV_MODULE_PAUSE 6

#define RTV_MODULE_FULL_SIZE 1
#define RTV_MODULE_QUARTER_SIZE 0

#define MAX_HOSTNAME_LENGTH 100

typedef struct
{
  int status;
  RTV_OBJECT *handle;
  RTV_ATTRIBUTES attributes;
  char *hostname;
  char *fileName;
}RTV_STRUCT;

static RTV_STRUCT rtv;
GC rtvGC = NULL;


char *GetHostname(void);

void InitializeRTV(void)
{
  rtv.status = RTV_MODULE_DISCONNECTED;
  rtv.handle = (RTV_OBJECT *)NULL;
  rtv.attributes.interlace = RTV_MODULE_FULL_SIZE;
  rtv.attributes.qfactor = GetQfactor();
  rtv.attributes.win_x = 0;
  rtv.attributes.win_y = 0;
  rtv.attributes.fps = 30;
  rtv.attributes.win_w = 640;
  rtv.attributes.win_h = 480;
  rtv.attributes.rate = GetBandwidth();
  rtv.attributes.audio_sample_rate = 48000;
  rtv.attributes.bytes_per_sample = 2;
  rtv.attributes.audio_channels = 2;
  rtv.attributes.audio_encoding = AUDIO_ENCODING_LINEAR;
  rtv.attributes.audio_endian = 0;
  rtv.hostname = (char *)NULL;
  rtv.fileName = (char *)NULL;

#ifndef USE_RTV_CODE
  PrintDiagnostics("Warning:  Diagnostic version - not using Rtv calls.");
#endif
}


int ConnectRTV(void)
{
  if(rtv.hostname != NULL)
  {
    free(rtv.hostname);
  }
  rtv.hostname = (char *)malloc(sizeof(char) * (strlen(GetHostname()) + 1));
  if(rtv.hostname == NULL)
  {
    PrintDiagnostics("Memory allocation error.  ConnectRTV failed");
    return(0);							    /* Error!  Could not allocate memory! */
  }

  strcpy(rtv.hostname, GetHostname());

#ifdef USE_RTV_CODE
  rtv.handle = (RTV_OBJECT *)RtvConnect(rtv.hostname);
  if(HandleRtvError()) return(0);
  RtvSetup(rtv.handle, SET_RTV_ATTRIBUTES, &rtv.attributes);
  if(HandleRtvError()) return(0);
#endif

  PrintDiagnostics("Rtv connected.");
  rtv.status = RTV_MODULE_CONNECTED;
  return(1);
}


int DisconnectRTV(void)
{
  if(rtv.fileName != NULL)
  {
#ifdef USE_RTV_CODE
    RtvCloseStream(rtv.handle);
/*    if(HandleRtvError()) return(0); */
#endif
    free(rtv.fileName);
    rtv.fileName = NULL;
  }
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, CLIENT_EXIT, 0);
/*  if(HandleRtvError()) return(0); */
  RtvDisconnect(rtv.handle);
  if(rtvGC != NULL)
  {
    XPlxUnsetRtvWindow(XtDisplay(recordWidget),
		       XtWindow(recordWidget),
		       rtvGC);
    XPlxUnsetRtvWindow(XtDisplay(playbackWidget),
		       XtWindow(playbackWidget),
		       rtvGC);
    XFreeGC(XtDisplay(recordWidget), rtvGC);
    rtvGC = NULL;
  }
#endif
  if(rtv.hostname != NULL)
  {
    free(rtv.hostname);
    rtv.hostname = NULL;
  }
  
  if(rtv.fileName != NULL)
  {
    free(rtv.fileName);
    rtv.fileName = NULL;
  }
  PrintDiagnostics("Rtv disconnected.");
  rtv.status = RTV_MODULE_DISCONNECTED;
  return(1);
}


int SetRTVFileName(char *fileName)
{
  switch(rtv.status)
  {
    case RTV_MODULE_CONNECTED:
    case RTV_MODULE_IDLE:
    {   
      break;
    }
      
    case RTV_MODULE_DISCONNECTED:
    {
      PrintDiagnostics("Warning!  RTV must be connected before you set the filename!");
      return(0);
      break;
    }
    
    case RTV_MODULE_PLAY:
    case RTV_MODULE_RECORD:
    case RTV_MODULE_PAUSE:
    {
      PrintDiagnostics("Warning!  RTV must be idle when you set the filename!");
      return(0);
      break;
    }
  }      
      
  /*
   * Is there a current filename?  If so, don't forget to close
   * the rtv stream and free the filename.
   */
  if(rtv.fileName != NULL)
  {
#ifdef USE_RTV_CODE
    RtvCloseStream(rtv.handle);
    if(HandleRtvError()) return(0);
#endif
    free(rtv.fileName);
    rtv.fileName = NULL;
  }
  rtv.fileName = (char *)malloc(sizeof(char) * (strlen(fileName) + 1));
  if(rtv.fileName == NULL)
  {
    /*
     * Error!  Could not allocate memory for the filename.
     */
    PrintDiagnostics("Error!  Memory allocation error.");
    return(0);
  }
  
  strcpy(rtv.fileName, fileName);

#ifdef USE_RTV_CODE
  RtvOpenStream(rtv.handle, rtv.fileName);  
  if(HandleRtvError()) return(0);
#endif
  rtv.status = RTV_MODULE_IDLE;
  return(1);
}


int SetRTVBandWidth(int bandwidth)
{
  char message[100];

  rtv.attributes.rate = bandwidth;
#ifdef USE_RTV_CODE
  RtvSetup(rtv.handle, SET_RTV_ATTRIBUTES, &rtv.attributes);  
  if(HandleRtvError()) return(0);
#endif
  sprintf(message, "Rtv bandwidth now set to %d Kb/field", bandwidth);
  PrintDiagnostics(message);
  return(1);
}

int SetRTVQfactor(int qfactor)
{
  char message[100];
  
  rtv.attributes.qfactor = qfactor;
#ifdef USE_RTV_CODE
  RtvSetup(rtv.handle, SET_RTV_ATTRIBUTES, &rtv.attributes);
  if(HandleRtvError()) return(0);
#endif
  sprintf(message, "Rtv minimum qfactor now set to %d.", qfactor);
  PrintDiagnostics(message);
  return(1);
}


int SetRTVfps(int fps)
{
  char message[100];
  
  if(fps != 0)
  {
    rtv.attributes.fps = fps;
  }
  else
  {
    rtv.attributes.fps = 30;
  }
  
#ifdef USE_RTV_CODE
  RtvSetup(rtv.handle, SET_RTV_ATTRIBUTES, &rtv.attributes);
  if(HandleRtvError()) return(0);
#endif
  sprintf(message, "Rtv frames per second now set to %d.", fps);
  PrintDiagnostics(message);
  return(1);
}

void SetRTVsize(int width, int height)
{
  char message[100];
  
  rtv.attributes.win_w = width;
  rtv.attributes.win_h = height;
  PrintDiagnostics(message);
}


void SetRTVPosition(int x, int y)
{

}


int RTVRecord(void)
{
#ifdef USE_RTV_CODE
  RTVWindowRecordPrepare();
  RtvRequest(rtv.handle, START_RTV_RECORD);
  if(HandleRtvError()) return(0);
#endif
  rtv.status = RTV_MODULE_RECORD;
  return(1);
}


int RTVPlay(void)
{
  switch(rtv.status)
  {
    case RTV_MODULE_CONNECTED:
    case RTV_MODULE_DISCONNECTED:
    case RTV_MODULE_RECORD:
    case RTV_MODULE_PLAY:
    case RTV_MODULE_PAUSE:
    {
      PrintError("Error!  Must be idle to call RTVPlay", ERROR_NONFATAL);
      return(0);
      break;
    }
  }      
    
#ifdef USE_RTV_CODE
  RTVWindowPlayPrepare();
  RtvRequest(rtv.handle, START_RTV_PLAY);
  if(HandleRtvError()) return(0);
#endif
  rtv.status = RTV_MODULE_PLAY;
  return(1);
}


int RTVPause(void)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, PAUSE_RTV_PLAY);
  if(HandleRtvError()) return(0);
#endif  
  return(1);
}

int RTVRecordPause(void)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, PAUSE_RTV_RECORD);
  if(HandleRtvError()) return(0);
#endif  
  return(1);
}


int RTVResume(void)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, RESUME_RTV_PLAY);
  if(HandleRtvError()) return(0);
#endif
  return(1);
}


int RTVRecordResume(void)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, RESUME_RTV_RECORD);
  if(HandleRtvError()) return(0); 
#endif
  return(1);
}



int RTVDisplayOneFrame(int currentFrame, int previousFrame, int fieldMode)
{
  int frameDistance;
  int minDistance;
  
  minDistance = 30;
  if(fieldMode == True)
  {
    minDistance *= 2;
  }
  
  if(GetMovieToolStatus() == MT_IDLE)
  {

    /*
     * RTV buffers 60 frames.  When we send a request to display
     * a specific frame, we could optimize it by realizing that
     * the frame is already in memory.  If the new frame is within
     * 30 frames of the last frame displayed, we would use
     * RTV_PLAY_SINGLE to look in memory for the specified frame.
     * If the desired frame is not within 30 frames of the current
     * frame, we should tell RTV to load it in from disk by using
     * RTV_SELECT_FRAME.
     */

    frameDistance = currentFrame - previousFrame;
    if(frameDistance < 0)
    {
      frameDistance *= -1;
    }
    
    if(frameDistance <= minDistance)
    {
#ifdef USE_RTV_CODE
      if(fieldMode != True)
      {
	rtv.handle->next_frame = currentFrame + rtv.attributes.first_valid_frame;
	RtvRequest(rtv.handle, RTV_PLAY_SINGLE);
	if(HandleRtvError()) return(0);
      }
      else
      {
	/*
	 * If fieldMode is set to True, we were given a field number
	 * instead of a frame number.
	 */
	rtv.handle->next_frame = currentFrame/2 + rtv.attributes.first_valid_frame;
	rtv.handle->next_field = (currentFrame % 2);
	RtvRequest(rtv.handle, RTV_SELECT_FIELD);
	if(HandleRtvError()) return(0);
      }
#endif
    }
    else
    {
#ifdef USE_RTV_CODE
      if(fieldMode != True)
      {
	rtv.handle->next_frame = currentFrame + rtv.attributes.first_valid_frame;
	rtv.handle->next_field = 0;
	RtvRequest(rtv.handle, RTV_SELECT_FRAME);
	if(HandleRtvError()) return(0);
      }
      else
      {
	/*
	 * If fieldMode is set to True, we were given a field number
	 * instead of a frame number.
	 */
	rtv.handle->next_frame = currentFrame/2 + rtv.attributes.first_valid_frame;
	rtv.handle->next_field = (currentFrame % 2);
	RtvRequest(rtv.handle, RTV_SELECT_FIELD);
	if(HandleRtvError()) return(0);
      }
#endif
    }
  }
  return(1);
}

int RTVDisplayOneFrameWithSlider(int currentFrame, int fieldMode)
{
  /*
   * Note:  Displaying a frame with the search slider is different in that
   * you could actually end up displaying more than 30 frames per second.
   * To make sure that the driver can handle the number of requests that
   * can possibly come out of a slider movement, another call was developed
   * in the RTV api.  Be sure to use this call instead, or the rtv driver
   * could crash.
   */
  if(GetMovieToolStatus() == MT_IDLE)
  {
#ifdef USE_RTV_CODE
    if(fieldMode != True)
    {
      rtv.handle->next_frame = currentFrame + rtv.attributes.first_valid_frame;
      rtv.handle->next_field = 0;
    }
    else
    {
      rtv.handle->next_frame = currentFrame/2 + rtv.attributes.first_valid_frame;
      rtv.handle->next_field = currentFrame%2;
    }
    RtvRequest(rtv.handle,RTV_SHUTTLE);
    if(HandleRtvError()) return(0);
#endif
  }
  return(1);
}


int RTVStop(void)
{
  switch(rtv.status)
  {
    case RTV_MODULE_PLAY:
    {
#ifdef USE_RTV_CODE
      RtvRequest(rtv.handle, STOP_RTV_PLAY);
      if(HandleRtvError()) return(0);
#endif
      rtv.status = RTV_MODULE_IDLE;
      break;
    }

    case RTV_MODULE_RECORD:
    { 
#ifdef USE_RTV_CODE 
      RtvRequest(rtv.handle, STOP_RTV_RECORD);  
      if(HandleRtvError()) return(0);
#endif
      rtv.status = RTV_MODULE_IDLE;
      break;
    }
    default:
    {
      PrintDiagnostics("Internal error!  Must be playing or recording to call RTVStop!");
      break;
    }
  }
  return(1);
}


int RTVOpenMovie(char *fileName, int *framesPerSecond, int *movieWidth, int *movieHeight)
{

  switch(rtv.status)
  {
    case RTV_MODULE_DISCONNECTED:
    case RTV_MODULE_PLAY:
    case RTV_MODULE_RECORD:
    case RTV_MODULE_PAUSE:
    {
      PrintDiagnostics("Internal error!  Cannot open a movie in current rtv state");
      return(0);
      break;
    }
  }

  SetRTVFileName(fileName);
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, INIT_RTV_PLAY);
  if(HandleRtvError()) return(0);

  RtvSetup(rtv.handle, GET_RTV_ATTRIBUTES, &rtv.attributes);
  if(HandleRtvError()) return(0);
  *framesPerSecond = rtv.attributes.fps;

  RtvSetup(rtv.handle, GET_RTV_PLAY_ATTRIBUTES, &rtv.attributes);

  RtvSetup(rtv.handle, GET_RTV_WINDOW, &rtv.attributes); 
  if(HandleRtvError()) return(0); 
  *movieWidth = rtv.attributes.win_w;
  *movieHeight = rtv.attributes.win_h;

#else
  *framesPerSecond = 25;
  *movieWidth = 320;
  *movieHeight = 240;
#endif
  return(1);
}


char *GetHostname(void)
{
  char command[100];
  char localHostname[100];
  static char *localHostnamePtr = NULL;
  char *tmpFileName;
  FILE *tmpFilePtr;
  int index;

  if(localHostnamePtr != NULL)
  {
    free(localHostnamePtr);
  }
  
  /* Check to see if there is an environment variable 'HOSTNAME' */
  if(getenv("LOCAL_HOSTNAME") != NULL)
  {
    localHostnamePtr = (char *)malloc(sizeof(char) * (strlen(getenv("LOCAL_HOSTNAME")) + 1));
    if(localHostnamePtr != NULL)
    {
      strcpy(localHostnamePtr, getenv("LOCAL_HOSTNAME"));
    }  
    return(localHostnamePtr);
  }
  
  /*
   * There was no environment variable, get local hostname through a system
   * call to 'hostname'.
   */
  tmpFileName = tmpnam(NULL);
  sprintf(command, "hostname > %s", tmpFileName);
  system(command);
  tmpFilePtr = fopen(tmpFileName, "r");
  fgets(localHostname, MAX_HOSTNAME_LENGTH, tmpFilePtr);
  remove(tmpFileName);
  
  /*
   * Remove the unwanted characters in the hostname string.
   */
  index = 0;
  while(index < strlen(localHostname))
  {
    if(localHostname[index] == '\n')
    {
      localHostname[index] = '\0';
    }
    index++;
  }
  localHostnamePtr = (char *)malloc(sizeof(char) * (strlen(localHostname) + 1));
  strcpy(localHostnamePtr, localHostname);
  return(localHostnamePtr);
}


int GetRTVMovieLength(int *length)
{
  /* 
   * Initialize the movie for playback.
   */
#ifdef USE_RTV_CODE
  RtvSetup(rtv.handle, GET_RTV_PLAY_ATTRIBUTES, &rtv.attributes);
  if(HandleRtvError()) return(0);
  *length = rtv.attributes.last_valid_frame - rtv.attributes.first_valid_frame;
#else
  *length = 1200;
#endif
  return(1);
}


int RTVGetCurrentFrame(int *frame)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, RTV_GET_CURRENT_FRAME);
  if(HandleRtvError()) return(0);
  *frame = rtv.handle->next_frame - rtv.attributes.first_valid_frame;
#else
  *frame = 0;
#endif
  return(1);
}


int RTVGetServerStatus(int *currentFrame)
{
  /*
   * Get the current status of the Rtv server.  If the
   * Rtv server is still functioning, the current frame
   * is returned in handle->next_frame.  If not, there
   * will be an error code.
   */
#ifdef USE_RTV_CODE
  if(HandleRtvError()) return(0);
  *currentFrame = rtv.handle->next_frame - rtv.attributes.first_valid_frame;
#endif
  if(*currentFrame < 0)
    return(0);
  else
    return(1);
}

int RTVAudioOff(void)
{
#ifdef USE_RTV_CODE
  rtv.handle->audio_mode = 0;
  RtvRequest(rtv.handle, RTV_AUDIO_MODE);
  if(HandleRtvError()) return(0);
#endif  
  return(1);
}

int RTVAudioOn(void)
{
  int audioAvailable;
  
  if(!RTVAudioAvailable(&audioAvailable))
  {
    PrintDiagnostics("Could not request audio status from the rtv server.");
    audioAvailable = False;
  }
  if(audioAvailable)
  {  
#ifdef USE_RTV_CODE
    rtv.handle->audio_mode = 1;
    RtvRequest(rtv.handle, RTV_AUDIO_MODE);
    if(HandleRtvError()) return(0);
#endif
  }
  return(1);
}

int RTVAudioAvailable(int *available)
{
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, RTV_AUDIO_STATUS);
  if(HandleRtvError()) return(0);
  *available = rtv.handle->audio_status;
#else
  *available = 1;
#endif
  return(1);
}


void RTVWindowPlayPrepare(void)
{
  Display *display;
  Window window;
  int width;
  int height;
  InputParameters inputParameters;

  display = XtDisplay(playbackWidget);
  window = XtWindow(playbackWidget);
  
  GetCurrentMovieSize(&width, &height);
  /*
   * Initialize RTV decompression via server protocol 
   * requests using XPlx API.
   */
#ifdef USE_RTV_CODE
  if(rtvGC != NULL)
  {
    GetInputParameters(&inputParameters);
    XPlxRtvPutCImage(display, window, rtvGC,
		     0, 0, width, height,
		     0 ,0, width, height,
		     inputParameters.inputSignalStandard);
  }
#endif    
}


int RTVRecordInit(void)
{
  SetRTVFileName((char*)GetMovieFileName());    
#ifdef USE_RTV_CODE
  RtvRequest(rtv.handle, INIT_RTV_RECORD);
  if(HandleRtvError()) return(0);
#endif
  return(1);
}

void RTVWindowRecordPrepare(void)
{
  Display *display;
  Window window;
  int width;
  int height;

  display = XtDisplay(recordWidget);
  window = XtWindow(recordWidget);
  
  GetLiveVideoSize(&width, &height);

  /*
   * Initialize RTV compression via server protocol 
   * requests using XPlx API.
   */
#ifdef USE_RTV_CODE
  if (rtvGC != NULL)
  {
    XPlxRtvGetCImage(display, window, rtvGC,
		     0, 0, width, height,
		     width, height);
  }
#endif
}

void RTVSetFirstValidFrame(int frame)
{
  rtv.attributes.first_valid_frame = frame + rtv.attributes.first_valid_frame; 
  RtvSetup(rtv.handle, SET_RTV_PLAY_ATTRIBUTES, &rtv.attributes);
}

void RTVSetLastValidFrame(int frame)
{
  rtv.attributes.last_valid_frame = frame + rtv.attributes.first_valid_frame; 
  RtvSetup(rtv.handle, SET_RTV_PLAY_ATTRIBUTES, &rtv.attributes);
}

int RTVUseOriginalMovieLength(void)
{
  int currentFrame;
  
  RtvRequest(rtv.handle, RTV_GET_FRAME_COUNT); 
  if(HandleRtvError()) return(0);

  rtv.attributes.first_valid_frame = 0;
  rtv.attributes.last_valid_frame = rtv.handle->frame_count;
  RtvSetup(rtv.handle, SET_RTV_PLAY_ATTRIBUTES, &rtv.attributes);
  RTVGetCurrentFrame(&currentFrame);
  return(currentFrame);
}


int HandleRtvError(void)
{
#ifndef DEBUG
  if(rtv.handle->errno < 0)
  {
    char errorMessage[200];
    
    /*
     * Error!
     */
    switch(rtv.handle->errno)
    {
      case RTV_ENOENT:
      {
	/*
	 * No such file.
	 */
	sprintf(errorMessage, "%s is not a valid rtv file.", GetMovieFileName());
	PrintError(errorMessage, ERROR_NONFATAL);
	ShowOpenFileDialog((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
	break;
      }
	
      case RTV_BADARG:
      {
	PrintError("Internal argument error.  Please restart RTMovieTool.", ERROR_NONFATAL);
	break;
      }
	
      case RTV_EIO:
      {
	/*
	 * I/O error.
	 */
	PrintError("I/O error.  See help for more details.", ERROR_NONFATAL);
	break;
      }

      case RTV_EAGAIN:
      {
	/*
	 * No Rtv server.
	 */
	PrintError("Rtv Connection Error.  Lost the connection to the Rtv server.  Exiting.", ERROR_FATAL);
	DisplayHelp(HELP_ERROR_RTV_CONNECTION);
	break;
      }
	
      case RTV_ENODEV:
      {
	/*
	 * No device driver.  Either the tvtwo driver or the ramdisk
	 * driver is not installed.
	 */
	PrintError("Driver error.  tvtwo driver not installed correctly.  Exiting.", ERROR_FATAL);
	DisplayHelp(HELP_ERROR_DRIVER);
	break;
      }
	
      case RTV_ENOSPC:
      {
	/*
	 * No space on device.
	 */
	PrintError("No space on device.", ERROR_NONFATAL);
	break;
      }

      case RTV_ECANCELED:
      {
	/*
	 * Operation canceled.
	 */
	int status;
	char operation[10];
	
	status = GetMovieToolStatus();
	switch(status)
	{
	  case MT_PLAY:
	  case MT_PLAY_PAUSE:
	  {
	    strcpy(operation, "Play");
	    break;
	  }
	  case MT_RECORD:
	  case MT_RECORD_PAUSE:
	  {
	    strcpy(operation, "Record");
	    break;
	  }
	}
	sprintf(errorMessage, "%s operation canceled.  Please try again.", operation);
	PrintError(errorMessage, ERROR_NONFATAL);
	break;
      }

      case RTV_END_FRAME:
      {
	/*
	 * End of movie.
	 */
	rtv.handle->errno = 0;
	SetMovieToolStatus(MT_IDLE);
	rtv.status = RTV_MODULE_IDLE;
	return(0);

	break;
      }
	
      case RTV_EOF_DETECT:
      {
	/*
	 * End of file detected.  Movie is truncated.
	 */
	PrintError("No end of file marker found - movie file has been truncated.", ERROR_NONFATAL);
	break;
      }
	
      default:
      {
	/*
	 * Bad error code.
	 */
	sprintf(errorMessage, "Internal error -- unknown error code: %d", rtv.handle->errno);
	PrintError(errorMessage, ERROR_NONFATAL);
      }
    }
    rtv.handle->errno = 0;
    switch(rtv.status)
    {
      case RTV_MODULE_PLAY:
      case RTV_MODULE_RECORD:
        RTVStop();
        break;
    }
    rtv.status = RTV_MODULE_IDLE;
    return(1);
  }
#endif
  return(0);
}
