/* $Header: /plx/source/Xclients/generic/RCS/structs.h,v 1.8 1996/01/30 20:44:38 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: structs.h,v $
 * Revision 1.8  1996/01/30  20:44:38  lovvik
 *   Added an image format field to the MovieTool
 *   structure.
 *
 * Revision 1.7  1996/01/11  18:13:16  lovvik
 *   Added OutputParameters to the MovieToolStruct to keep track
 *   of the current output format and standard.
 *
 * Revision 1.6  1995/11/15  03:01:41  lovvik
 * Added defines for error codes.
 *
 * Revision 1.5  1995/11/01  02:41:32  lovvik
 * Changed SyncDelay to 1000.  (Was 5000).
 *
 * Revision 1.4  1995/10/12  20:05:28  lovvik
 * Added a mode field to MovieToolStruct.  Also, I added
 * defines for frame mode and field mode.
 *
 * Revision 1.3  1995/10/11  17:00:02  lovvik
 * Added a #define for NO_SERVER_SUPPORT.  This is used
 * in conditional compilations until Rtv is fully integrated
 * into the server.
 *
 * Revision 1.2  1995/10/06  17:08:07  lovvik
 * Commented the #define DEBUG statement (making RTMovieTool
 * compile Rtv code).
 *
 * Revision 1.1  1995/10/02  23:47:24  lovvik
 * Initial revision
 * */

static char movieToolStructsRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/structs.h,v 1.8 1996/01/30 20:44:38 lovvik Exp $";

/* #define DEBUG */
#ifndef DEBUG
#define USE_RTV_CODE
#endif


/*
 * Structures for RTMovieTool:
 */

/*-------------------------------- MovieTool dialog -------------------------*/
typedef struct
{
  int channelToUse;
  int inputSignalStandard;
  int inputSignalFormat;
  int inputInstance;
  int inputWidth;
  int inputHeight;
} InputParameters;

typedef struct
{
  int outputSignalStandard;
  int outputSignalFormat;
  int outputSignalWidth;
  int outputSignalHeight;
} OutputParameters;


typedef struct
{
  int width;
  int height;
} FrameSize;


typedef struct
{
  int status;
  int liveVideo;
  int movieLoaded;
  char* movieFileName;
  int movieLengthInFrames;
  int framesPerSecond;
  int currentFrame;
  int outputStatus;
  int inputStatus;
  int outputAvailable;
  int mode;
  char imageFormat[20];
  InputParameters inputParameters;
  OutputParameters outputParameters;
  FrameSize liveVideoSize; 
  FrameSize currentMovie;
}  MovieToolStruct;

#define NO_MOVIE 1
#define MT_IDLE 2
#define MT_PLAY_PAUSE 3
#define MT_RECORD_PAUSE 4
#define MT_PLAY 5
#define MT_RECORD 6
#define FULL_SIZE 1
#define QUARTER_SIZE 2
#define FRAME_MODE 1
#define FIELD_MODE 2

#define SynchDelay 1000

/*
 * Error codes:
 */
#define NO_ERROR 0
#define ERROR_NONFATAL 1
#define ERROR_FATAL 2
/*---------------------------------------------------------------------------*/



/*----------------------------------- Timer ---------------------------------*/
typedef struct
{
  int hours;
  int minutes;
  int seconds;
  int frames;
} TimerStruct;
/*---------------------------------------------------------------------------*/



/*------------------------ Compression preferences dialog -------------------*/
typedef struct
{
  int width;
  int height;
  int bandwidth;
  int qfactor;
} CompressionStruct;

/*---------------------------------------------------------------------------*/



/*------------------------ Audio dialog -------------------------------------*/
typedef struct
{
  int playback;
  int record;
  int monitor;
} AudioVolumeStruct;

typedef struct
{
  int outputDevice;
  int inputDevice;
  int monitorUserSetting;
  int monitorAppSetting;
  int audioOn;
  AudioVolumeStruct audioVolume;
} AudioStruct;

#define SPEAKER		1
#define HEADPHONES	2
#define LINE_OUT	4
#define LINE_IN		8
#define MICROPHONE	16
#define ON		True
#define OFF		False

/*---------------------------------------------------------------------------*/


/*------------------------ Color settings dialog ----------------------------*/
typedef struct
{
  int brightness;
  int contrast;
  int hue;
  int saturation;
} ColorSettingsStruct;
/*---------------------------------------------------------------------------*/


typedef struct
{
  Widget widget;
  XtPointer client_data;
  XtPointer xt_call_data;
} ControlStruct;

#define HELP_ERROR_RTV_CONNECTION 8
#define HELP_ERROR_DRIVER 9
