/* $Header: /plx/source/Xclients/generic/RCS/window.c,v 1.9 1996/01/30 20:46:14 lovvik Exp $ */

/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: window.c,v $
 * Revision 1.9  1996/01/30  20:46:14  lovvik
 *   Fixed the logic that keeps track of the
 *   state of the play and record windows (mapped
 *   or unmapped).
 *
 * Revision 1.8  1996/01/11  18:14:59  lovvik
 *   Changed the code that repositions output.  The new output
 *   function is StartOutput();
 *
 * Revision 1.7  1996/01/04  18:06:45  lovvik
 * Added event handling code to the movie shell.  This allows output
 * to track the playback window.
 *
 * Revision 1.6  1995/11/15  02:45:57  lovvik
 * Corrected a problem with output.  I made the output rectangle track
 * the playback window instead of the record window.
 *
 * Revision 1.5  1995/11/01  02:48:57  lovvik
 * Added some code to position the main shell and the record window on
 * startup.  Also added some code to determine whether or not the record
 * and playback windows are mapped.  (These windows must be mapped before
 * calling RTVWindowRecordPrepare() or RTVWindowPlayPrepare()).
 *
 * Revision 1.4  1995/10/12  20:33:01  lovvik
 * Added some conditional code until the server is fully integrated
 * with Rtv.  The definition is SOME_SERVER_SUPPORT, and is found in
 * structs.h.
 *
 * Revision 1.3  1995/10/11  17:05:23  lovvik
 * Removed an include statement for "InputCascade.h".  This
 * include file no longer exists.  (Now it is called "IOCascade.h").
 * This header file is not needed in this module.
 *
 * Revision 1.2  1995/10/06  17:20:49  lovvik
 * Removed some code that was necessary until we had server
 * support for Rtv.
 *
 * Revision 1.1  1995/10/02  23:49:21  lovvik
 * Initial revision
 * */

static char movieToolRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/window.c,v 1.9 1996/01/30 20:46:14 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleBG.h>

#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>
#include <stdio.h>

#include "globals.h"
#include "interface.h"
#include "structs.h"
#include "prototypes.h"

#define SIZEOFIMAGE 1024*1024*512
#define WM_BORDER_WIDTH 10

Boolean InitializeVideoWorkProc(XtPointer unusedClientData);

static int playWindowMapped = False;
static int recordWindowMapped = False;

void InitializeWindowModule(void)
{
  XtAddEventHandler(input1Shell,
		    StructureNotifyMask,
		    False,
		    RTVWindowMovementHandler,
		    (XtPointer)NULL);

  XtAddEventHandler(recordWidget,
		    StructureNotifyMask,
		    False,
		    RTVWindowMovementHandler,
		    (XtPointer)NULL);

  XtAddEventHandler(movieShell,
		    StructureNotifyMask,
		    False,
		    RTVWindowMovementHandler,
		    (XtPointer)NULL);

  XtAddEventHandler(playbackWidget,
		    StructureNotifyMask,
		    False,
		    RTVWindowMovementHandler,
		    (XtPointer)NULL);

  XtAddEventHandler(playbackWidget,
		    ExposureMask,
		    False,
		    RTVWindowExposeHandler,
		    (XtPointer)NULL);

  XtAddEventHandler(recordWidget,
		    ExposureMask,
		    False,
		    RTVWindowExposeHandler,
		    (XtPointer)NULL);
}




void RTVWindowMovementHandler(Widget widget,
				    XtPointer clientData,
				    XEvent* event,
				    Boolean* continueToDispatch)
{
  int clientX;
  int clientY;
  int screenNumber;
  Window rootWindow;
  Window recordWindow;
  Display *display;
  int  newWidth;
  int  newHeight;
  
  switch(event->type)
  {
    case ConfigureNotify:
    {
      clientX = 0;
      clientY = 0;
      display = XtDisplay(recordWidget);
      recordWindow = XtWindow(recordWidget);
      screenNumber = XDefaultScreen(display);
      rootWindow = RootWindow(display, screenNumber);

      if(widget == recordWidget)
      {
	/*
	 * Has the window size changed?
	 */
	newWidth = event->xconfigure.width;
	newHeight = event->xconfigure.height;
	SetLiveVideoSize(newWidth, newHeight, False);
      }

      if(widget == playbackWidget)
      {
	newWidth = event->xconfigure.width;
	newHeight = event->xconfigure.height;
	SetCurrentMovieSize(newWidth, newHeight);  
	if(GetOutputStatus() == True)
	{
	  StartOutput();
	}
      }

      if(widget == movieShell)
      {
	if(GetOutputStatus() == True)
	{
	  StartOutput();
	}
      }
      break;      
    }
    case MapNotify:
    {
      if(widget == movieShell)
      {
	playWindowMapped = True;
      }
      if(widget == input1Shell)
      {
	recordWindowMapped = True;
      }
      break;
    }  

    case UnmapNotify:
    {
      if(widget == movieShell)
      {
	playWindowMapped = False;
      }
      if(widget == input1Shell)
      {
	recordWindowMapped = False;
      }
      break;
    }  
  }
}



void RTVWindowExposeHandler(Widget widget, 
			    XtPointer clientData,
			    XEvent* event,
			    Boolean* continueToDispatch)
{
  switch(event->type)
  {
    case Expose:
    {
      switch(GetMovieToolStatus())
      {
	case MT_PLAY_PAUSE:
	case MT_RECORD_PAUSE:
	case MT_PLAY:
	case MT_RECORD:
        {
	  if(widget == playbackWidget)
	  {
	    RTVWindowPlayPrepare();
	  }	    
	  if(widget == recordWidget)
	  {
	    RTVWindowRecordPrepare();
	  }	    
	  break;
	}
	  
	case MT_IDLE:
        {
	  if(widget == playbackWidget)
	  {
	    /*
	     * If we have a movie loaded AND we are idle, we need to display
	     * the new frame.
	     */
	    if(GetMovieLoaded())
	    {
	      RTVWindowPlayPrepare();
	      RefreshCurrentFrame();
	    }
	  }
	  
	  else
	  {

	  }
	  break;
	}
      }
    }      
  }
}



int IsPlayWindowMapped(void)
{
  return(playWindowMapped);
}


int IsRecordWindowMapped(void)
{
  return(recordWindowMapped);
}


void GetRootWindowSize(int *width, int *height)
{
  Display *display;
  Window rootWindow;
  XWindowAttributes attributes;
  
  display = XtDisplay(mainWindow);
  rootWindow = XDefaultRootWindow(display);
  XGetWindowAttributes(display, rootWindow, &attributes);
  *width = attributes.width;
  *height = attributes.height;
}


void GetMainWindowSize(int *width, int *height)
{
  Display *display;
  Window window;
  XWindowAttributes attributes;
  
  display = XtDisplay(mainWindow);
  window = XtWindow(mainWindow);
  XGetWindowAttributes(display, window, &attributes);
  *width = attributes.width;
  *height = attributes.height;
}

void PositionWindows(void)
{
  int rootWidth;
  int rootHeight;
  int mainWindowWidth;
  int mainWindowHeight;
  
  GetRootWindowSize(&rootWidth, &rootHeight);
  GetMainWindowSize(&mainWindowWidth, &mainWindowHeight);

  XtMoveWidget(mainWindow, rootWidth-(mainWindowWidth + WM_BORDER_WIDTH), 0);
  XtMoveWidget(input1Shell, 0, 0);
}
