/*
 *   Copyright (c) 1995 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/* $Log: rtvPlay.c,v $
 * Revision 1.4  1996/01/31  15:51:22  lovvik
 *   Added code to make rtvPlay start at the first
 *   valid frame.  Also, changed the error detection
 *   to look for negative values only.
 *
 * Revision 1.3  1996/01/18  18:40:45  lovvik
 *   Removed an extraneous call to XPlxRtvPutCImage().
 *
 * Revision 1.2  1996/01/11  17:01:59  lovvik
 *   Added support for the XVideoVIO card.  This includes
 *   the following change:
 *     If the video card is an XVideoVIO, set the first
 *     input to PAL.
 *
 *   Also made a correction that allows rtvPlay to play a
 *   movie with no video source connected.  This involves
 *   performing an XPlxVideoInputSelect() using the same standard
 *   as that of the movie.
 *
 * Revision 1.1  1995/11/29  00:42:58  lovvik
 *   Initial revision
 * */

/* $Header: /plx/source/Xclients/generic/RCS/rtvPlay.c,v 1.4 1996/01/31 15:51:22 lovvik Exp $ */

static char rtvPlayRcsid[] = "$Header: /plx/source/Xclients/generic/RCS/rtvPlay.c,v 1.4 1996/01/31 15:51:22 lovvik Exp $";

#include <stdio.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>
#include <rtv_ops.h>

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 256
#endif

void Control_C_Handler(int sig);
Boolean XVideoCard(Display *display, Window window, GC videoGC);

Widget			top, video_widget;
plx_IO*			plxio = NULL;
plx_signal*		plxsignal = NULL;
int			movieWidth = 640;
int			movieHeight = 512;
int			done;
static 			Atom deleteWindowAtom;

void main(argc, argv)
     int argc;
     char **argv;
{
  Window		window;
  Display*		display;
  int			scr;
  int			mask;
  int			movieSignalStandard;
  int			movieFPS;
  int			inputChannel = PLX_INPUT_0;
  GC			rtvGC;
  GC			videoGC;
  XEvent		event;
  XSetWindowAttributes	attr;
  int			depth;
  XVisualInfo*		visualInfo;
  int			firstExpose = True;
  RTV_OBJECT		*handle;
  RTV_ATTRIBUTES	attributes;
  char                  hostname[MAXHOSTNAMELEN];
  
  done = False;

  if(argc != 2)
  {
    fprintf(stderr, "Must supply path and filename on the command line.\n");
    fprintf(stderr, "ex:  rtvPlay /home/movie\n");
    exit(0);
  }

  if(!XPlxIsRTVFile(argv[1]))
  {
    /*
     * Invalid Rtv file.
     */
    fprintf(stderr, "Error!  %s is not a valid rtv file.\n", argv[1]);
    exit(0);
  }


  /*
   * Catch Control-C.  Note:  Do NOT leave the Rtv
   * server in a play state when the client exits.
   */
  signal(SIGINT, Control_C_Handler);

  /*
   * Connect your client program with the X server.
   */
  display = XOpenDisplay(getenv("DISPLAY"));

  /*
   * Get a default screen.
   */
  scr = (int) XDefaultScreen(display);

  /*
   * Get a proper visual for using 24-bit video in a window.
   */
  visualInfo = XPlxGetVideoVisual(scr, display);
  depth = visualInfo->depth;

  /*
   * If the video visual is a true color window, we must create
   * a color map for it.
   */
  if (TrueColor == visualInfo->class)
  {
    attr.colormap = XCreateColormap(display, XRootWindow(display, scr),
				    visualInfo->visual, AllocNone);
  }
  else
  {
    attr.colormap = XDefaultColormap(display, scr);
  }

  /*
   * WARNING!!! Must set up the colormap, the background pixel, and the
   * border pixel to make it work when the window depth is different from
   * the root depth.
   */
  mask = CWBackPixel | CWColormap | CWBorderPixel;
  attr.background_pixel = BlackPixel(display, DefaultScreen(display));
  attr.border_pixel = WhitePixel(display, DefaultScreen(display));

  /*
   * Create the drawable for the XPlx calls.
   */
  window = XCreateWindow(display, RootWindow(display, scr),
			 0, 0, movieWidth, movieHeight, 1, depth,
			 InputOutput, visualInfo->visual, mask, &attr);

  /*
   * Catch exposure events.
   */
  XSelectInput(display, window, ExposureMask);

  rtvGC = XCreateGC(display, window, 0, 0);
  videoGC = XCreateGC(display, window, 0, 0);
  XFlush(display);

  /*
   * Connect to the Rtv server.
   */
  gethostname(hostname, MAXHOSTNAMELEN);
  handle = (RTV_OBJECT*)RtvConnect(hostname);
  if(handle == NULL)
  {
    fprintf(stderr, "Error!  Could not connect to the Rtv server.\n");
    exit(0);
  }

  if(XVideoCard(display, window, videoGC) == True)
  {
    /*
     * This is an XVideoVIO or an XVideoRGB card.  We must set the first
     * input to PAL mode.
     */
    XPlxVideoInputSelect(display, window, videoGC, 
			 PLX_INPUT_0, 
			 PLX_PAL,
			 PLX_COMP,
			 0);
    inputChannel = PLX_INPUT_1;
  }

  /*
   * It is not necessary to set up the attributes for
   * playback.  The attributes are used only for recording.
   * For a recording operation, set up the attributes
   * here.  (See RtvSetup()).
   */

  /*
   * Open a play session.
   */
  RtvOpenStream(handle, argv[1]);
  
  RtvRequest(handle, INIT_RTV_PLAY);
  RtvSetup(handle, GET_RTV_ATTRIBUTES, &attributes);
  RtvSetup(handle, GET_RTV_WINDOW, &attributes);
  RtvSetup(handle, GET_RTV_PLAY_ATTRIBUTES, &attributes);
  
  movieWidth = attributes.win_w;
  movieHeight = attributes.win_h;
  movieFPS = attributes.fps;

  /*
   * Based on the fps, we can determine a compatible
   * signal standard.
   */
  switch(movieFPS)
  {
    case 30:
    {
      movieSignalStandard = PLX_NTSC;
      break;
    }
    case 25:
    {
      movieSignalStandard = PLX_PAL;
      break;
    }
  }      

  /*
   * Indicate to the window server that this is an Rtv window.
   */
  XPlxSetRtvWindow(display, window, rtvGC);

  /*
   * Set up the input to generate interrupts for this movie.
   * NOTE:  The standard that you pass into XPlxVideoInputSelect()
   * must be the same as that of the movie.
   */
  XPlxVideoInputSelect(display, window, videoGC, 
		       inputChannel,
		       movieSignalStandard,
		       PLX_COMP,
		       0);

  /*
   * Prepare the window for playing the movie.
   */
  XResizeWindow(display, window, movieWidth, movieHeight);

  XMapWindow(display, window); 

  /*
   * Catch the Delete Window call from the window manager.
   */
  deleteWindowAtom = XInternAtom(display,
				 "WM_DELETE_WINDOW",
				 False);
  XSetWMProtocols(display, window, &deleteWindowAtom, 1);
  while (1)
  {
    if(XPending(display))
    {
      XNextEvent(display, &event);
      switch(event.type)
      {
	case Expose:
        {
	  if(firstExpose == True)
	  {
	    /*
	     * Indicate to the window server that this is an Rtv window
	     * to be used for playing a movie.  XPlxRtvPutCImage() must
	     * be called AFTER the first expose event.
	     */
	    XPlxRtvPutCImage(display, window, rtvGC, 0, 0,
			     movieWidth, movieHeight, 0, 0,
			     movieWidth, movieHeight,
			     movieSignalStandard);

	    /*
	     * In this simple example, we will have the
	     * first expose event trigger the play operation.
	     * Don't forget to start at the first valid
	     * frame in the movie.
	     */
	    handle->next_frame = attributes.first_valid_frame;
	    RtvRequest(handle, RTV_SELECT_FRAME);
	    RtvRequest(handle, START_RTV_PLAY);
	    firstExpose = False;
	  }
	  break;
	}

	case ClientMessage:
	{
	  /*
	   * Catch the delete window message from the window
	   * manager.
	   */
	  if(event.xclient.data.l[0] == deleteWindowAtom)
	  {
	    done = True;
	  }
	  break;
	}
      }	/* end of switch */
    }

    /*
     * When the movie is finished playing, the Rtv server sets handle->errno
     * to RTV_END_FRAME.  This is a non-fatal error, but we can use it here
     * to detect the end of the movie and exit the client.
     */
    if(handle->errno < 0 || done == True)
    {
      /*
       * The movie is finished playing.  Disconnect from the Rtv server and
       * indicate to the window server that it does not need to forward
       * window configuration changes to the Rtv server.
       */
      RtvRequest(handle, STOP_RTV_PLAY);
      handle->errno = 0;
      RtvCloseStream(handle);
      RtvRequest(handle, CLIENT_EXIT, 0);
      RtvDisconnect(handle);
      XPlxUnsetRtvWindow(display, window, rtvGC);

      XFreeGC(display, rtvGC);
      exit(0);
    }
  }
} /* end program */


void Control_C_Handler(int sig)
{
  signal(SIGINT, Control_C_Handler);
  done = True;
}

Boolean XVideoCard(Display *display, Window window, GC videoGC)
{
  plx_IO 	*hardwareInfo;
  plx_IO_list 	*inputPtr;
  plx_IO_list 	*outputPtr;
  int 		channel;
  
  int    hardwareCode = 0;

  /*
   * This function determines if you are using an XVideo
   * VIO card based on the number of input and output
   * channels.  Two input channels and one output channel
   * indicates that you are using an XVideoVIO card.
   */
  hardwareInfo = XPlxQueryConfig(display, window, videoGC);
  if(hardwareInfo == NULL)
  {
    /*
     * This is not a Parallax video card!
     */
    return(False);
  }
  
  /*
   * Count the input channels:
   */
  inputPtr = hardwareInfo->inputs;
  while(inputPtr != NULL)
  {
    channel = inputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= (1 << (channel - 1));
    inputPtr = inputPtr->next_IO;
  }

  /*
   * Count the output channels:
   */
  outputPtr = hardwareInfo->outputs;
  while(outputPtr != NULL)
  {
    channel = outputPtr->channel;
    /*
     * Get rid of the instance information.
     */
    channel &= 0x00FF;

    hardwareCode |= ((1 << channel) << 8);
    outputPtr = outputPtr->next_IO;
  }

  /*
   * hardwareCode:
   *                       0xFFFF
   *                          ^ ^
   *   Indicate the outputs  /   \ Indicate the inputs.
   */

  return(hardwareCode == 0x0103);
}

